package com.gic.enterprise.interceptor;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.dubbo.rpc.RpcContext;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import com.alibaba.fastjson.JSON;
import com.gic.enterprise.ano.IgnoreLogin;
import com.gic.enterprise.base.MenuInfo;
import com.gic.enterprise.constants.Constants;
import com.gic.enterprise.context.UserContext;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.exception.CommonException;
import com.gic.enterprise.utils.UserDetail;
import com.gic.enterprise.utils.UserDetailUtils;


public class AuthInterceptor extends HandlerInterceptorAdapter {

    private static final Logger log = LogManager.getLogger(AuthInterceptor.class);

    private static final Map<String, Object> ignoreUriBeforeLoginMap = new HashMap<>(16);
    private static final Map<String, Object> ignoreUriAfterLoginMap = new HashMap<>(16);
    static {
        ignoreUriBeforeLoginMap.put("/gic-auth-web/login", "1");
        ignoreUriBeforeLoginMap.put("/gic-auth-web/logout", "1");
        ignoreUriBeforeLoginMap.put("/gic-auth-web/list-enterprise-by-phone", "1");
        ignoreUriAfterLoginMap.put("/gic-auth-web/login-user-menu", "1");
        ignoreUriAfterLoginMap.put("/gic-auth-web/login-user-operation-item", "1");
        ignoreUriBeforeLoginMap.put("/api-plug/dict-district-list", "1");
        ignoreUriBeforeLoginMap.put("/api-plug/upload-images", "1");
        ignoreUriBeforeLoginMap.put("/api-plug/query-provinces", "1");
        ignoreUriBeforeLoginMap.put("/api-plug/query-city", "1");
        ignoreUriBeforeLoginMap.put("/api-plug/query-county", "1");
        ignoreUriBeforeLoginMap.put("/api-plug/query-district", "1");
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object object) throws Exception {
        String token = UserContext.getContext().getToken();
        RpcContext.getContext().getAttachments().put(Constants.USER_TOKEN, token);
        System.out.println(JSON.toJSONString(RpcContext.getContext().getArguments()));

        // 更新redis
        UserContext.getContext().refresh();
        if (!(object.getClass().isAssignableFrom(HandlerMethod.class))) {
            return true;
        }
        HandlerMethod handler = (HandlerMethod) object;
        //免登录接口路由
        IgnoreLogin ignoreLogin = handler.getMethodAnnotation(IgnoreLogin.class);
        if (null != ignoreLogin) {
            return true;
        }

        //获取uri
        String uri = request.getRequestURI();
        //操作项标志位
        String isControl = request.getHeader("isControl");
        //是否是操作项请求
        boolean isOperationItemRequest = StringUtils.isNotBlank(isControl);
        log.info("用户接口访问标志：{}", isControl);
        log.info("用户接口访问路由：{}", uri);
        //不需要拦截的路径
        if (ignoreUriBeforeLoginMap.containsKey(uri)) {
            return true;
        }
        //判断是否有权限的路径
        UserDetail userDetail = UserDetailUtils.getUserDetail();
        if (userDetail == null || userDetail.getUserInfo() == null) {
            throw new CommonException(ErrorCode.LOGIN_INVALID.getErrorCode(), "登录信息不存在，请登录");
        }

        //不需要拦截的路径
        if (ignoreUriAfterLoginMap.containsKey(uri)) {
            return true;
        }
        //增删改，即操作项类型的操作，需要 获取操作模块，一级、二级的ID和名称
        if (isOperationItemRequest) {
            List<MenuInfo> menuInfoList = userDetail.getMenuInfoList();
            Map<String, MenuInfo> moduleUrlMap = userDetail.getModuleUrlMap();
            //页面路径
            String moduleUrl = request.getHeader("Referer");
            log.info("完整路由：{}", moduleUrl);
            //project_url_for_web + menu_url
            moduleUrl = getModuleUrl(moduleUrl);
            log.info("操作模块的路由：{}", moduleUrl);
            //获取当前操作项数据
            MenuInfo menuInfo = moduleUrlMap.get(moduleUrl);
            if (menuInfo != null) {
                Map<String, MenuInfo> tempMap = listToMap(menuInfoList);

                Integer temp = menuInfo.getMenuId();
                //获取第二层级的页面
                if (menuInfo.getLevel().intValue() > 2) {
                    temp = getOperationModule(menuInfo.getMenuId(), tempMap);
                    while (tempMap.get(temp.toString()).getLevel().intValue() != 2) {
                        temp = getOperationModule(tempMap.get(temp.toString()).getMenuId(), tempMap);
                    }
                }
                int secondLevelMenuId = 0;
                String secondLevelMenuName = "无";
                int firstLevelMenuId;
                String firstLevelMenuName;
                //如果是第二级的
                if (menuInfo.getLevel().intValue() == 2) {
                    secondLevelMenuId = tempMap.get(temp.toString()).getMenuId();
                    secondLevelMenuName = tempMap.get(temp.toString()).getMenuName();
                    //获取第一层级的页面
                    temp = getOperationModule(tempMap.get(temp.toString()).getMenuId(), tempMap);
                    firstLevelMenuId = tempMap.get(temp.toString()).getMenuId();
                    firstLevelMenuName = tempMap.get(temp.toString()).getMenuName();
                } else {
                    //第一级
                    firstLevelMenuId = tempMap.get(temp.toString()).getMenuId();
                    firstLevelMenuName = tempMap.get(temp.toString()).getMenuName();
                }
                log.info("一级页面的ID和名称:{},{}", firstLevelMenuId, firstLevelMenuName);
                log.info("二级页面的ID和名称:{},{}", secondLevelMenuId, secondLevelMenuName);
                request.setAttribute("moduleMenuId", secondLevelMenuId);
                request.setAttribute("moduleMenuName", firstLevelMenuName + "-" + secondLevelMenuName);
            }
        }

        //超级管理员不限制
        if (userDetail.getUserInfo().getSuperAdmin().intValue() == 1) {
            return true;
        }
        //判断是否有权限
        Map<String, Object> menuUrlMap = userDetail.getMenuUrlMap();
        if (menuUrlMap == null || menuUrlMap.isEmpty()) {
            throw new CommonException(ErrorCode.SYSTEM_ERROR.getErrorCode(), "sorry，您无该页面的访问权限，请联系超级管理员！");
        }

        if (isOperationItemRequest) {
            //说明是操作项
            //如果匹配上了，说明是没权限
            if (menuUrlMap.containsKey(uri)) {
                throw new CommonException(ErrorCode.SYSTEM_ERROR.getErrorCode(), "sorry，您无该页面的访问权限，请联系超级管理员！");
            }
        } else {
            //说明是页面
            //如果没有匹配上，说明没权限
            if (!menuUrlMap.containsKey(uri)) {
                throw new CommonException(ErrorCode.SYSTEM_ERROR.getErrorCode(), "sorry，您无该页面的访问权限，请联系超级管理员！");
            }
        }

        return true;
    }

    private static Integer getOperationModule(Integer menuId, Map<String, MenuInfo> tempMap) {
        MenuInfo temp = tempMap.get(menuId.toString());
        if (temp != null) {
            return temp.getParentId();
        }
        return null;
    }

    private static Map<String, MenuInfo> listToMap(List<MenuInfo> list) {
        Map<String, MenuInfo> map = new HashMap<>(16);
        if (CollectionUtils.isNotEmpty(list)) {
            for (MenuInfo menuInfo : list) {
                map.put(menuInfo.getMenuId().toString(), menuInfo);
            }
        }
        return map;
    }

    /**
     * 截取域名后面的路径
     * @param url
     * @return
     */
    private static String getModuleUrl(String url) {
        //形如https://four.gicdev.com/damo-system/account-manage/create-admin
        String regex = "/";
        String[] stringArr = url.split(regex);
        //去掉0、1、2
        StringBuilder result = new StringBuilder();
        for (int i = 3, length = stringArr.length; i < length; i ++) {
            if (i == length - 1) {
                //最有一个，需要去掉参数，即？后面的东西
                String temp = stringArr[i];
                result.append(regex).append(temp.split("[?]")[0]);
            } else {
                result.append(regex).append(stringArr[i]);
            }
        }
        return result.toString();
    }
}
