package com.gic.download.utils;

import java.io.*;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletResponse;

import com.gic.download.constants.ExcelExtensionEnum;
import com.gic.thirdparty.BucketNameEnum;
import com.gic.thirdparty.FileOperateUtils;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.client.utils.DateUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;

public class DownloadUtils {
    private static final Logger LOGGER = LogManager.getLogger(DownloadUtils.class);

    private static final String SUFFIX_EXCEL_2007 = "xlsx";
    private static final String SUFFIX_CSV = "csv";
    private static final String SUFFIX_EXCEL_2003 = "xls";


    /**
     * 分页下载数据获取类
     * @author Silver
     * @date 2017年3月16日 上午11:45:13
     *
     * @param <T>
     *
     */
    protected abstract class DownloadDataLoader<T> {

        /**
         * 分页下载属性值控制操作类
         * @param bean
         * @param propertyName
         * @param property
         * @return
         * @author Silver
         * @date 2017年3月16日 上午11:45:45
         */
        protected String convertProperty(T bean, String propertyName, Object property) {
            return property == null ? "" : property.toString();
        }

        /**
         * 分页下载属性赋值
         * @param bean
         * @param propertyName
         * @return
         * @author Silver
         * @date 2017年3月16日 上午11:46:37
         */
        protected Object getProperty(T bean, String propertyName) {
            try {
                return BeanUtils.getProperty(bean, propertyName);
            } catch (Throwable e) {
                LOGGER.info("bean:" + bean + ",Property:" + propertyName + e.getMessage(), e);
                return null;
            }
        }

        /**
         * 数据获取接口
         * @param pageNum -- 从0计数
         * @return
         * @throws Exception
         * @author Silver
         * @date 2017年3月16日 上午11:47:07
         */
        protected abstract List<T> getDownloadData(Integer pageNum) throws Exception;
    };


    protected static interface Writer {
        public void write(Collection<String> row) throws IOException;
    }

    /**
     * Web下载文件
     * @param tempPath 临时路径
     * @param reportId 报表中心ID
     * @param fileName 文件名称
     * @param excelExtensionCode Excel文件扩展名 枚举 ExcelExtensionEnum
     * @param headerList 表头名称
     * @param propertyNameList 字段名
     * @param loader    数据加载类
     * @param needEncryptField 需要加密的字段，每一个元素存的是字段，如phone（电话）
     * @param columnWidth 列宽
     * @throws Exception
     * @author Silver
     * @date 2017年3月16日 上午11:47:31
     */
    protected <T> void download(String tempPath, Integer reportId, String fileName, Integer excelExtensionCode, List<String> headerList, List<String> propertyNameList, DownloadDataLoader<T> loader, List<String> needEncryptField, List<Integer> columnWidth) throws Exception {

        if (StringUtils.isEmpty(fileName) || loader == null || CollectionUtils.isEmpty(propertyNameList)) {
            throw new RuntimeException("参数错误。FileName:" + fileName + ",DataLoader:" + loader + ",PropertyName:" + propertyNameList);
        }
        // 获取输出流,设置content-type等头域
        //final OutputStream out = getResponseStream(response ,fileName);
        File tempFile = null;
        //重置临时文件路径
        DataDownloadUtils.mkDir(new File(tempPath));
        File file = new File(tempPath);
        //初始化数据
        tempFile = File.createTempFile(fileName, ExcelExtensionEnum.getExtensionByCode(excelExtensionCode), file);
        final OutputStream out = new FileOutputStream(tempFile);
        try {
            Writer writer = null;
            // 获取文件后缀名
            String extension = ExcelExtensionEnum.getExtensionByCode(excelExtensionCode);
            // 如果是excel的后缀
            if (ExcelExtensionEnum.EXCEL_2003.getExtension().equalsIgnoreCase(extension) || ExcelExtensionEnum.EXCEL_2007.getExtension().equalsIgnoreCase(extension)) {
                Workbook workbook = new HSSFWorkbook();
                Sheet sheet = workbook.createSheet("sheet1");
                final List<Collection<String>> rows = new ArrayList<>();
                writer = new Writer() {
                    @Override
                    public void write(Collection<String> row) {
                        rows.add(row);
                    }
                };

                writeOutputStream(loader, propertyNameList, writer, needEncryptField);
                // 写入excel
                if (!ExcelUtils.setExcelInfo(sheet, columnWidth, headerList, rows)) {
                    throw new IOException("设置导出文件内容失败。");
                }

                workbook.write(out);
            } else if (ExcelExtensionEnum.CSV.getExtension().equalsIgnoreCase(extension)) {
                writer = new Writer() {
                    @Override
                    public void write(Collection<String> row) throws IOException {
                        String str = ExcelUtils.collectionToCsvString(row);
                        byte[] content = org.apache.commons.codec.binary.StringUtils.getBytesUnchecked(str + "\n",
                                "GBK");
                        IOUtils.write(content, out);
                        out.flush();
                    }
                };
                // 写文件头
                writer.write(headerList);
                // 写文件
                writeOutputStream(loader, propertyNameList, writer, needEncryptField);
            } else {
                writer = new Writer() {
                    @Override
                    public void write(Collection<String> row) throws IOException {
                        IOUtils.write(org.apache.commons.codec.binary.StringUtils.getBytesUnchecked(row + "\n",
                                "GBK"), out);
                        out.flush();
                    }
                };
                // 写文件头
                writer.write(headerList);
                // 写文件
                writeOutputStream(loader, propertyNameList, writer, needEncryptField);
            }
            out.flush();
            tempFile.deleteOnExit();
            //上传文件
            String fieldCode = "/" + System.currentTimeMillis() + "_" + fileName + ExcelExtensionEnum.getExtensionByCode(excelExtensionCode);
            //文件存进腾讯云
            String url = FileOperateUtils.simpleUploadFileFromLocal(tempFile, fieldCode, BucketNameEnum.REPORT_50000.getName());
            if (org.apache.commons.lang.StringUtils.isNotBlank(url)) {
                url = "https://" + url;
            }
            LOGGER.info("上传文件到腾讯云，路径：" + url);
            //更新数据
            DataDownloadUtils.uploadFile(reportId, url);
        } finally {
            IOUtils.closeQuietly(out);
        }
    }

    /**
     * 获得输出流
     *
     * @return
     * @throws IOException
     */
    protected OutputStream getResponseStream(HttpServletResponse response, String fileName) throws IOException {
        if (StringUtils.isEmpty(fileName)) {
            fileName = DateUtils.formatDate(new Date(), "yyyy-MM-dd_HH-mm-ss") + SUFFIX_CSV;
        }
        response.reset();

        String extension = FilenameUtils.getExtension(fileName);
        if(SUFFIX_EXCEL_2007.equalsIgnoreCase(extension)){
            // 部分window版本生成后的xlsx打不开，默认改成xls打开
            fileName = fileName.substring(0, fileName.length() -1);
        }
        //设置响应编码
        response.setCharacterEncoding("UTF-8");
        //设置对应的contentType
        response.setContentType("application/x-download;charset=UTF-8");
        response.setHeader("Content-Disposition", "attachment; filename="
                + new String(fileName.getBytes("gb2312"), "ISO-8859-1"));
        OutputStream out = response.getOutputStream();


        return out;
    }

    protected <T> void writeOutputStream(DownloadDataLoader<T> loader, List<String> propertyNames,
                                         Writer writer, List<String> needEncryptField) throws Exception {

        int pageNum = 1;
        int maxLength = 102400;
        while (maxLength -- > 0) {
            // 分页获取数据
            List<T> objList = null;
            try {
                objList = loader.getDownloadData(pageNum++);
            } catch (Exception e) {
                LOGGER.error("获得到处数据异常:{}",e.getMessage(), e);
            }
            if (CollectionUtils.isEmpty(objList)) {
                break;
            }
            for (T bean : objList) {
                if (bean == null) {
                    continue;
                }
                Collection<String> result = new ArrayList<String>();
                // 遍历指定属性
                for (String name : propertyNames) {
                    // 获得属性值
                    Object property = loader.getProperty(bean, name);
                    // 将属性值转换成字符串
                    String convertValue = loader.convertProperty(bean, name, property);
                    //加密处理
                    if (needEncryptField != null && needEncryptField.contains(name)) {
                        convertValue = encrypt(convertValue);
                    }
                    // 组装成row
                    result.add(convertValue);
                }
                if (CollectionUtils.isEmpty(result)) {
                    continue;
                }
                writer.write(result);
            }
        }

    }

    private static String encrypt(String data) {
        if (StringUtils.isBlank(data)) {
            return data;
        }
        //手机号或者会员卡号 第5-8位用*代替
        int length = data.length();
        if (length > 8) {
            return data.substring(0, 4) + "****" + data.substring(8);
        } else if (length > 4) {
            return data.substring(0, 4) + "****";
        } else {
            return "****";
        }
    }

}
